@@ -240,8 +240,8 @@ class AtlasClass(object):
                         else:
                             msg = 'Datum {} had an empty string for key: {}'.format(datum, key)
                             raise ValueError(msg)
-
-                if not isinstance(datum[key], (str, float, int)):
+                import math
+                if not isinstance(datum[key], (str, float, int)) or isinstance(datum[key], math.nan):
                     raise Exception(
                         f"Metadata sent to Atlas must be a flat dictionary. Values must be strings, floats or ints. Key `{key}` of datum {str(datum)} is in violation."
                     )
@@ -500,9 +500,24 @@ class AtlasProjection:
                 neighbors: A set of ids corresponding to the nearest neighbors of each query
                 distances: A set of distances between each query and its neighbors
         '''
+
         if queries is None and ids is None:
             raise ValueError('You must specify either a list of datum `ids` or numpy array of `queries` but not both.')
 
+        max_k = 128
+        max_queries = 256
+        if k > max_k:
+            raise Exception(f"Cannot query for more than {max_k} nearest neighbors. Set `k` to {max_k} or lower")
+
+        if ids is not None:
+            if len(ids) > max_queries:
+                raise Exception(f"Max ids per query is {max_queries}. You sent {len(ids)}.")
+        if queries is not None:
+            if not isinstance(queries, np.array):
+                raise Exception("`queries` must be an instance of np.array.")
+            if queries.shape[0] > max_queries:
+                raise Exception(f"Max vectors per query is {max_queries}. You sent {queries.shape[0]}.")
+
         if self.project.is_locked:
             raise ValueError("Your map cannot be queried for nearest neighbors while it builds. Check the dashboard to see your map builds progress.")
 
@@ -533,7 +548,7 @@ class AtlasProjection:
         status = response.status_code
 
         if response.status_code == 500:
-            raise AssertionError('Cannot perform vector search on your map at this time. Try again later.')
+            raise Exception('Cannot perform vector search on your map at this time. Try again later.')
 
         if response.status_code != 200:
             raise Exception(response.json()['detail'])
@@ -1470,3 +1485,27 @@ class AtlasProject(AtlasClass):
                     label_to_datums[label] = []
                 label_to_datums[label].append(item['datum_id'])
         return label_to_datums
+
+    def add_tags(self) -> Dict[str, str]:
+        '''
+        Retrieves back all tags made in the web browser for a specific project and map.
+
+        Returns:
+            A dictionary mapping datum ids to tags.
+        '''
+        # now get the tags
+        datums_and_tags = requests.post(
+            self.atlas_api_path + '/v1/project/tag/read/all_by_datum',
+            headers=self.header,
+            json={
+                'project_id': self.id,
+            },
+        ).json()['results']
+
+        label_to_datums = {}
+        for item in datums_and_tags:
+            for label in item['labels']:
+                if label not in label_to_datums:
+                    label_to_datums[label] = []
+                label_to_datums[label].append(item['datum_id'])
+        return label_to_datums
